/*
   This program creates the homotopy between a circle and a square
   but it computes the points on the circle and square a bit differently.
   This version pre-computes all of the points on the circle and the square
   (instead of just the points with angles between 0 and Pi/4).

   Notice that computing the points for the square is more difficult than
   computing the points for the circle.
*/
#include <windows.h>
#include <math.h>
#include "glut.h"
#include "glui.h"

#define N 200          // make N divisible by 8
#define PI 3.14159265
double circleX[N], circleY[N], squareX[N], squareY[N];

/* GLUT callback function prototypes */
void display(void);
void reshape(int, int);
void slider(float);

int windowHeight;         // this is set by the glut reshape() callback function
int windowWidth;          // this is set by the glut reshape() callback function

#define sliderInit 0.0
float sliderValue = sliderInit;  // this is set by the glui slider() callback function

int main(int argc, char** argv)
{
   int i;
   int window;
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);  // glui needs double buffering!
   // specify the screen window
   glutInitWindowSize(500,500);       // width, height (in pixels)
   glutInitWindowPosition(100,150);   // left, top (in pixels)
   window = glutCreateWindow("Homotopy v4");
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   /* set up viewing */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(-2.0, 2.0, -2.0, 2.0); /* viewing volume */
   glMatrixMode(GL_MODELVIEW);

   // parent window, x, y, width, height, initial value, callback function
   gluiHorizontalSlider(window, 10, 0, -10, 20, sliderInit, slider);

   glClearColor(1.0, 1.0, 1.0, 0.0);
   glLineWidth(1.0);

   // pre-compute circle points up to the angle 2*Pi
   for (i = 0; i < N; i++)
   {
      circleX[i] = cos((i/(double)N)*2*PI);
      circleY[i] = sin((i/(double)N)*2*PI);
      squareX[i] = 1.0;
      squareY[i] = tan((i/(double)N)*2*PI);
   }

   // pre-compute square points up to the angle Pi/4 (45 degrees)
   for (i = 0; i <= N/8; i++)
   {
      squareX[i] = 1.0;
      squareY[i] = tan((i/(double)(N/8))*PI/4.0);
   }

   // pre-compute square points for angles Pi/4 to Pi/2
   for (i = N/8; i < N/4; i++)
   {
      squareX[i] = squareY[N/4-i];
      squareY[i] = 1.0;
   }

   // pre-compute square points for angles Pi/2 to 3*Pi/4
   for (i = N/4; i < 3*N/8; i++)
   {
      squareX[i] = -squareY[i-N/4];
      squareY[i] = 1.0;
   }

   // pre-compute square points for angles 3*Pi/4 to Pi
   for (i = 3*N/8; i < N/2; i++)
   {
      squareX[i] = -1.0;
      squareY[i] = squareY[N/2-i];
   }

   // pre-compute square points for angles Pi to 5*Pi/4
   for (i = N/2; i < 5*N/8; i++)
   {
      squareX[i] = -1.0;
      squareY[i] = -squareY[i-N/2];
   }

   // pre-compute square points for angles 5*Pi/4 to 3*Pi/2
   for (i = 5*N/8; i < 3*N/4; i++)
   {
      squareX[i] = -squareY[3*N/4 - i];
      squareY[i] = -1.0;
   }

   // pre-compute square points for angles 3*Pi/3 to 7*Pi/4
   for (i = 3*N/4; i < 7*N/8; i++)
   {
      squareX[i] = squareY[i-3*N/4];
      squareY[i] = -1.0;
   }

   // pre-compute square points for angles 7*Pi/4 to 2*Pi
   for (i = 7*N/8; i < N; i++)
   {
      squareX[i] = 1.0;
      squareY[i] = -squareY[N-i];
   }

   glutMainLoop( );

   return 0;
}//main()


void display(void)
{
   int i;
   double X, Y;
   float t = sliderValue;

   // specify the viewport
   glViewport( 0,  0, windowWidth, windowHeight); // left, bottom, width, height (in pixels)

   glClear( GL_COLOR_BUFFER_BIT );

   glColor3f(0.0f, 1.0f, 0.0f);

   // draw the radial "homotopy lines"
   glBegin(GL_LINES);
      for(i = 0; i < N; i++)
      {
         glVertex2f(circleX[i], circleY[i]);
         glVertex2f(squareX[i], squareY[i]);
      }
   glEnd();

   glColor3f(1.0f, 0.0f, 0.0f);

   // draw the circle
   glBegin(GL_LINE_STRIP);
      for(i = 0; i < N; i++)
      {
         glVertex2f(circleX[i], circleY[i]);
      }
   glEnd();

   // draw the square
   glBegin(GL_LINE_STRIP);
      for(i = 0; i < N; i++)
      {
         glVertex2f(squareX[i], squareY[i]);
      }
   glEnd();

   glColor3f(0.0f, 0.0f, 1.0f);

   // draw the curve interpolated between the circle and the square
   glBegin(GL_LINE_STRIP);
      for(i = 0; i < N; i++)
      {
         X = (1-t)*circleX[i]+t*squareX[i];  // lerp the x-coordinate
         Y = (1-t)*circleY[i]+t*squareY[i];  // lerp the y-coordinate
         glVertex2f(X, Y);
      }
   glEnd();

   glutSwapBuffers();  // glui needs double buffering!
}


/* The reshape() callback function */
void reshape(int width, int height)
{
   windowWidth  = width;
   windowHeight = height;

   printf("We got a \"reshape(%d, %d)\" event\n", width, height);

   gluiReshape(width, height);  // notice the difference here
}//reshape()


/* glui slider callback function */
void slider(float value)
{
   sliderValue = value;

   printf("We got a \"slider(%f)\" event\n", value);

   glutPostRedisplay();  // notice the difference here
}//slider()